@@ -29,7 +29,6 @@
             [ring.util.codec :as codec]
             [clojure.string :as str]
             [clojure.tools.logging :as log]
-            [puppetlabs.kitchensink.core :as ks]
             [puppetlabs.trapperkeeper.services.webserver.jetty9-config :as config]
             [schema.core :as schema]))
 
@@ -129,30 +128,6 @@
 (def RegisteredEndpoints
   {schema/Str [Endpoint]})
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Constants
-
-(def default-graceful-stop-timeout
-  "The default number of milliseconds that is allowed for requests active at
-  the time the server is stopped to remain running until the server shuts
-  down."
-  60000)
-
-(def default-proxy-idle-timeout
-  "The default number of milliseconds to wait before the proxy gives up on the
-  upstream server if the :idle-timeout value isn't set in the proxy config."
-  60000)
-
-(def default-queue-idle-timeout
-  "The maximum number of milliseconds that a thread in the queue can remain
-  idle before the thread may be thrown away.  A value less than or equal to 0
-  would allow a thread to remain idle indefinitely."
-  60000)
-
-(def default-queue-min-threads
-  "The minimum number of threads to create and maintain in the queue."
-  8)
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Utility Functions
 
@@ -240,11 +215,17 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Jetty Server / Connector Functions
 
+(defn- http-configuration
+  [request-header-size]
+  (let [http-config (doto (HttpConfiguration.)
+                      (.setSendDateHeader true))]
+    (if request-header-size
+      (.setRequestHeaderSize http-config request-header-size))
+    http-config))
+
 (defn- connection-factories
   [request-header-size ssl-ctxt-factory]
-  (let [http-config (doto (HttpConfiguration.)
-                      (.setSendDateHeader true)
-                      (.setRequestHeaderSize request-header-size))
+  (let [http-config (http-configuration request-header-size)
         factories   (into-array ConnectionFactory
                                 [(HttpConnectionFactory. http-config)])]
     (if ssl-ctxt-factory
@@ -252,6 +233,10 @@
         ssl-ctxt-factory factories)
       factories)))
 
+(defn- thread-count
+  [setting]
+  (if setting setting -1))
+
 (schema/defn ^:always-validate
   connector* :- ServerConnector
   [server :- Server
@@ -262,12 +247,13 @@
         connector   (doto (ServerConnector.
                             server
                             nil nil nil
-                            (config/acceptors-count (ks/num-cpus))
-                            (config/selectors-count (ks/num-cpus))
+                            (thread-count (:acceptor-threads config))
+                            (thread-count (:selector-threads config))
                             (connection-factories request-size ssl-ctxt-factory))
                       (.setPort (:port config))
-                      (.setHost (:host config))
-                      (.setSoLingerTime (:so-linger-milliseconds config)))]
+                      (.setHost (:host config)))]
+    (when-let [linger-millis (:so-linger-milliseconds config)]
+      (.setSoLingerTime connector linger-millis))
     (when-let [idle-timeout (:idle-timeout-milliseconds config)]
       (.setIdleTimeout connector idle-timeout))
     connector))
@@ -287,25 +273,61 @@
   (connector* server config nil))
 
 (schema/defn ^:always-validate
-  queue-thread-pool :- QueuedThreadPool
-  [max-threads :- schema/Int
-   queue-max-size :- schema/Int]
-  (let [queue-min-size (min default-queue-min-threads queue-max-size)]
-    (QueuedThreadPool. max-threads
-                       queue-min-size
-                       default-queue-idle-timeout
-                       (BlockingArrayQueue.
-                         queue-min-size
-                         queue-min-size
-                         queue-max-size))))
+  queue-thread-pool :- (schema/maybe QueuedThreadPool)
+  [max-threads :- (schema/maybe schema/Int)
+   queue-max-size :- (schema/maybe schema/Int)]
+  (if (or max-threads queue-max-size)
+    (let [thread-pool (if max-threads
+                        (QueuedThreadPool. max-threads)
+                        (QueuedThreadPool.))]
+      (if queue-max-size
+        ;; The code below is definitely not ideal, but there isn't a way to set
+        ;; the maximum capacity of the QueuedThreadPool's BlockingArrayQueue
+        ;; after construction.  We're trying to avoid hard-coding our own
+        ;; defaults for other settings that we want Jetty to control, e.g., the
+        ;; initial capacity of the queue and minimum number of threads.  By
+        ;; reconstructing the QueuedThreadPool here, we can use Jetty's defaults
+        ;; for settings unrelated to `queue-max-size`.
+        ;;
+        ;; QueuedThreadPool and BlockingArrayQueue construction isn't too
+        ;; expensive.  It mostly involves some initial memory allocations.  The
+        ;; more expensive work - where threads are actually started and the
+        ;; queue expands to fulfill new requests - would only happen after the
+        ;; QueuedThreadPool were started.  That won't happen for the
+        ;; `thread-pool` instance from above, which just gets thrown away as
+        ;; this function falls out of scope without ever having been started.
+        ;; Also, this function would likely only be called once per server
+        ;; startup where a `queue-max-size` were configured.
+        ;;
+        ;; The QueuedThreadPool constructor sets the `queue-capacity` and
+        ;; `queue-grow-by` based on the minimum number of threads available
+        ;; in the pool.  See https://github.com/eclipse/jetty.project/blob/jetty-9.2.10.v20150310/jetty-util/src/main/java/org/eclipse/jetty/util/thread/QueuedThreadPool.java#L92-L96.
+        ;; That algorithm is essentially duplicated here, with the only
+        ;; difference being that if `queue-max-size` is smaller than the
+        ;; minimum number of threads, the `queue-capacity` and `queue-grow-by`
+        ;; are reduced to the `queue-max-size` in order to avoid the
+        ;; BlockingArrayQueue constructor throwing an IllegalArgumentException.
+        (let [min-threads    (.getMinThreads thread-pool)
+              queue-capacity (min queue-max-size min-threads)
+              queue-grow-by  (min queue-max-size min-threads)]
+          (QueuedThreadPool. (.getMaxThreads thread-pool)
+                             min-threads
+                             (.getIdleTimeout thread-pool)
+                             (BlockingArrayQueue.
+                               queue-capacity
+                               queue-grow-by
+                               queue-max-size)))
+        thread-pool))))
 
 (schema/defn ^:always-validate
   create-server :- Server
   "Construct a Jetty Server instance."
   [webserver-context :- ServerContext
    config :- config/WebserverConfig]
-  (let [server (Server. (queue-thread-pool (:max-threads config)
-                                           (:queue-max-size config)))]
+  (let [server (if-let [pool (queue-thread-pool (:max-threads config)
+                                                (:queue-max-size config))]
+                 (Server. pool)
+                 (Server.))]
     (when (:jmx-enable config)
       (let [mb-container (MBeanContainer. (ManagementFactory/getPlatformMBeanServer))]
         (doto server
@@ -431,19 +453,19 @@
                                                    @(:state webserver-context))]
                          (HttpClient. ssl-ctxt-factory)
                          (HttpClient.)))]
-          (if request-buffer-size
-            (.setRequestBufferSize client request-buffer-size)
-            (.setRequestBufferSize client config/default-request-header-buffer-size))
+          (when request-buffer-size
+            (.setRequestBufferSize client request-buffer-size))
           client))
 
       (createHttpClient []
         (let [client (proxy-super createHttpClient)
-              timeout (if idle-timeout (* 1000 idle-timeout)
-                                       default-proxy-idle-timeout)]
+              timeout (when idle-timeout
+                        (* 1000 idle-timeout))]
           (if (:follow-redirects options)
             (.setFollowRedirects client true)
             (.setFollowRedirects client false))
-          (.setIdleTimeout client timeout)
+          (when timeout
+            (.setIdleTimeout client timeout))
           client))
 
       (customizeProxyRequest [proxy-req req]
@@ -451,7 +473,7 @@
          (callback-fn proxy-req req)))
 
       ;; The implementation of onResponseFailure is duplicated heavily from:
-      ;; https://github.com/eclipse/jetty.project/blob/jetty-9.2.8.v20150217/jetty-proxy/src/main/java/org/eclipse/jetty/proxy/AbstractProxyServlet.java#L576-L607
+      ;; https://github.com/eclipse/jetty.project/blob/jetty-9.2.10.v20150310/jetty-proxy/src/main/java/org/eclipse/jetty/proxy/AbstractProxyServlet.java#L576-L607
       ;; The only significant difference is that a 'failure-callback-fn', if
       ;; defined in options, is invoked just prior to completing the async
       ;; context for cases that the response was not already committed upstream.
@@ -589,7 +611,8 @@
                                  (.setHandler maybe-logged))
                                maybe-logged)]
       (.setHandler s statistics-handler)
-      (.setStopTimeout s (or shutdown-timeout default-graceful-stop-timeout))
+      (when shutdown-timeout
+        (.setStopTimeout s shutdown-timeout))
       (when-let [script (:post-config-script options)]
         (config/execute-post-config-script! s script))
       (assoc webserver-context :server s))))
